﻿local lm = CLootMaster		-- Local instance of the addon

local L = LibStub("AceLocale-3.0"):GetLocale("CouncilLootMaster")

function CLootMaster:Debug( message, verbose )
	if not self.debug then return end
	if verbose and not self.verbose then return end
	self:Print("debug: " .. message)
end

--[[
	Sends a command and attached message to specified target,
	using raid and party channels whenever possible
]]
function CLootMaster:SendCommand(command, message, target)
	if not target then
		return self:Print(L["Could not send command, no target specified"])
	end

	local formatted = format("%s:%s", tostring(command), tostring(message))
	local broadcasted = false

	if CLootMaster:UnitInRaid(target) then
		-- we're in raid with master looter
		self:SendCommMessage("CLootMasterML", formatted, "RAID", nil, "ALERT")
		broadcasted = true
		self:Debug('SendCommand(RAID): '..formatted, true)
	elseif UnitInParty(target) and GetNumSubgroupMembers()>0 then
		--we're in party with master looter
		self:SendCommMessage("CLootMasterML", formatted, "PARTY", nil, "ALERT")
		broadcasted = true
		self:Debug('SendCommand(PARTY): '..formatted, true)
	else
		--we're not grouped, send message to target by whispering
		self:SendCommMessage("CLootMasterML", formatted, "WHISPER", target, "ALERT")
		self:Debug('SendCommand(WHISPER->'..target..'): '..formatted, true)
	end

	-- Speedup messages to self by just calling the ML CommandReceived function.
	if CLootMasterML and target == UnitName("player") then
		local distribution = 'WHISPER'
        if broadcasted then
            distribution = 'RAID'
        end
		CLootMasterML.CommandReceived(CLootMasterML, nil, formatted, distribution, target)
	end
end

--[[
	Event gets triggered when candidate receives a message from the Masterlooter
]]
function CLootMaster:CommandReceived(prefix, message, distribution, sender)
	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '')
	message = message or ''

	if command == 'DO_YOU_WANT' then

		-- Masterlooter wants to know from us if we'd like to have the item.
		-- Lets show the gui and ask the player for input.

		local itemID, ilevel, binding, slot, quality, timeout, itemLink, texture, notesAllowed, eligibleClasses = strsplit("^", message)

		eligibleClasses = CLootMaster:DecodeUnlocalizedClasses(tonumber(eligibleClasses))

		local _, playerClass = UnitClass("player")

		if not eligibleClasses or (playerClass and not eligibleClasses[playerClass]) then
			self:Print(format(L['Autopassing %s (not eligible)'], itemLink or 'unknown item'))
			return
		end

		notesAllowed = ((tonumber(notesAllowed or 0) or 0) == 1)
		-- Send the master looter our current gear
		self:SendCommand( 'GEAR', format('%s^%s', itemID, self:GetGearByINVTYPE(slot, 'player')), sender)

		if not self:HasLoot(itemLink) then
			-- add the loot to the lootlist and redraw the ui
			tinsert( self.lootList, {
				["lootmaster"]      = sender,
				["link"]            = itemLink,
				["id"]              = itemID,
				["notesAllowed"]    = notesAllowed,
				["ilevel"]          = ilevel,
				["binding"]         = binding,
				["slot"]            = slot,
				["texture"]         = texture,
				["timeout"]         = tonumber(timeout),
				["timeoutLeft"]     = tonumber(timeout),
				["quality"]         = tonumber(quality)
			})

			self.lootMLCache[itemLink] = sender

			self:UpdateLootUI()
		end

	elseif command == 'LOOTED' then
		-- Someone looted an item through lootmaster.
		-- Update the UI

		-- Message gets received
		local player, itemLink, lootType = strsplit("^", message)

		self:Debug('looted: ' .. message)

		if self.lootMLCache[itemLink] and self.lootMLCache[itemLink] ~= sender then
			return self:Print(format('%s sent a message that %s has been looted, but is not lootmaster for this item (his version outdated?). Message ignored', sender, itemLink))
		end
		self.lootMLCache[itemLink] = nil

		if self:RemoveLoot(itemLink) then
			self:UpdateLootUI()
		end

		self:Debug('UI updated')

		if not player or not itemLink then
			return self:Debug('!player or !link')
		end

		lootTypeID = tonumber(lootType) or CLootMaster.LOOTTYPE.UNKNOWN
		lootType = CLootMaster.LOOTTYPE[lootTypeID] or CLootMaster.LOOTTYPE[CLootMaster.LOOTTYPE.UNKNOWN]

	elseif command == 'ENCHSKILL' then

		-- masterlooter wants to know if we have sufficient enchanting skill
		if self:GetEnchantingSkill() >= tonumber(message) then
			self:SendCommand('ENCHRESP', true, sender)
		end

	end
end

--[[
	Checks whether the loot list already contains an item, if so it returns the index of it
]]
function CLootMaster:HasLoot( item )
	for i, data in ipairs(self.lootList) do repeat
		if not data then break end
		if data.link == item or data.id == item then
			return i
		end
	until true end
	return nil
end

--[[
	Returns an items entry in the loot table
]]
function CLootMaster:GetLoot( item )
	local index = self:HasLoot(item)
	if not index then return nil end
	return self.lootList[index]
end

--[[
	Removes an item from the loot table
]]
function CLootMaster:RemoveLoot( item )
	local index = self:HasLoot(item)
	if not index then return false end
	tremove(self.lootList, index)
	return true
end

--[[
	Send a repsonse for an item to the master looter
]]
function CLootMaster:SendItemWanted( lootmaster, itemID, response, note)
	-- Just whisper the response back to the ml.
	local _, avgItemLevel = GetAverageItemLevel()
	self:SendCommand("WANT", format("%s^%s^%s^%s", itemID or '', response or 0, floor(avgItemLevel + 0.5), gsub(note or '','%^','')), lootmaster)
	local itemLink = self:GetLoot( itemID ).link
	self:Print( 'Responded ' .. CLootMaster.RESPONSE[response].TEXT .. ' to ' .. itemLink or 'unknown item' )
end

--[[
	Get the enchantingSkill as number (0 if not enchanter)
]]
function CLootMaster:GetEnchantingSkill()
	if _G.GetProfessions~=nil then
		local p1, p2 = GetProfessions()
		if p1==nil and p2==nill then return 0 end
		local enchNameLocalized = GetSpellInfo(7411) -- Enchanting - Apprentice
		if not enchNameLocalized then return 0 end
		if p1~=nil then
			local skillName, _, skillRank = GetProfessionInfo(p1)
			if skillName == enchNameLocalized then return skillRank end
		end
		if p2~=nil then
			local skillName, _, skillRank = GetProfessionInfo(p2)
			if skillName == enchNameLocalized then return skillRank end
		end
		return 0
	end

	local numSkills = GetNumSkillLines()
	local enchNameLocalized = GetSpellInfo(7411) -- Enchanting - Apprentice
	if not enchNameLocalized then return 0 end
	for i=1, numSkills do
		local skillName, _, _, skillRank = GetSkillLineInfo(i)
		if skillName == enchNameLocalized then return skillRank end
	end
	return 0
end

--[[
	Kind of singleton, just create the frame once.
]]
function CLootMaster:GetLootFrame()
	if self.lootframe then return self.lootframe end
	return 'test'
end
